




					/* SET YOUR TABS TO 4 TO READ THIS  */



#include "jpinfo.h"

#define VERSION "0.1"

/* modify this line if you prefer a different default language */
#define DEFAULT_LANGUAGE		"eng"


#define PROCESS_PACKETS			1000
#define CSA_PACKETS				70


#define MAX_PMT_SIZE			4096	// really have no idea, anyone?
#define MAX_SUBTITLE_CHANNELS	256		// ?
#define MAX_AUDIO_CHANNELS		256		// ?
#define MAX_TELETEXT_CHANNELS	256		// ?
#define MAX_AC3_CHANNELS		256		// ?


int process_packets, csa_packets;
unsigned char *input_buffer, *output_buffer;
int program_number;
int ecm_pid, pmt_pid, video_pid;

int subtitle_pid[MAX_SUBTITLE_CHANNELS];
int subtitle_channels;

int audio_pid[MAX_AUDIO_CHANNELS];
int audio_channels;
int selected_audio_pid;

int teletext_pid, ac3_pid;

int pass_all_flag;
int pmt_complete_flag;
unsigned char pmt_buffer[MAX_PMT_SIZE];
char language[4];

struct passwd *userinfo;
char *home_dir;
char *user_name;
int scrambled_flag;

int have_pids_flag;
int info_only_flag;
int use_ac3_pid_only_flag;


struct ECMINFO
{
  struct ECMINFO *next;
  char *name;
  int system;
  int ecm_pid;
  int id;
};


static struct ECMINFO *ecminfos = 0;
static struct ECMINFO *pa;

int debug = 0;

static volatile int stop = 0;


char *strsave(char *s) /* save char array s somewhere */
{
char *p;

p = malloc( strlen(s) +  1);
if(!p) return 0;

strcpy(p, s);

return p;
} /* end function strsave */


#define SECA_CA_SYSTEM			0x0100
#define VIACCESS_CA_SYSTEM		0x0500
#define IRDETO_CA_SYSTEM		0x0600
#define XXX_SYSTEM				0x0900 // 09 19 ???
#define CONAX_CA_SYSTEM			0x0b00
#define NAGRA_BEV_SYSTEM		0x1234
#define BETA_CA_SYSTEM			0x1700
#define NAGRA_CA_SYSTEM			0x1800
#define SHL_SYSTEM				0x4A00
#define CRYPTOWORKS_SYSTEM		0x0D00


/**
 * Parse CA descriptor
**/
static void ParseCADescriptor (uint8_t *data, int length)
{
static char *seca_name = "Seca";
static char *viaccess_name = "Viaccess";
static char *nagra_name = "Nagra";
static char *irdeto_name = "Irdeto";
static char *conax_name = "Conax";

int ca_system;
int j;

struct ECMINFO *e;

//fprintf(stderr, "POSITION=%d\n", data - pmt_buffer);

if(debug)
	{
//    fprintf(stderr,"ParseCADescriptor(): arg *data=%p length=%d\n", data, length);

//    for(j = 0; j < length; j++) fprintf(stderr," %02d", j);
//	fprintf(stderr,"\n");
//   for(j = 0; j < length; j++) fprintf(stderr," %02x", data[j]);
//    fprintf(stderr,"\n");

	fprintf(stderr, "pmt_buffer:\n");
	for(j = 0; j < 25; j++) fprintf(stderr, "%03d ", j + data - pmt_buffer);
	fprintf(stderr, "\n");
	for(j = 0; j < 25; j++) fprintf(stderr, " %02x ", pmt_buffer[j + data - pmt_buffer]);
	fprintf(stderr, "\n");
	}


/* only test the upper 8 bits of the ca_system */
ca_system = (data[0] << 8) /* | data[1] */;

//fprintf(stderr, "length=%d\n", length);

//fprintf(stderr, "CA system=0x%04x\n",  (data[0] << 8) | data[1]);
 

// lincardemu:
// 0x71 00 seca1?
// 0x10 00 seca2?

switch(ca_system)
	{
	case SECA_CA_SYSTEM:
    	for(j = 2; j < length; j += 15)
			{
			e = malloc(sizeof(struct ECMINFO));
			if (e == 0) return;
			e->system = ca_system;
			e->name = seca_name;
			e->ecm_pid = ((data[j] & 0x1f) << 8) | data[j+1];
			e->id = (data[j+2] << 8) | data[j+3];
			e->next = ecminfos;
			ecminfos = e;
			fprintf(stderr, "Found seca id %04x\n", e->id);
			}        
		break;

	case VIACCESS_CA_SYSTEM:
		j = 4;
		while(j < length)
			{
			if(data[j] == 0x14)
				{
				e = malloc(sizeof(struct ECMINFO));
				if (e == 0) return;
				e->system = ca_system;
				e->name = viaccess_name;
				e->ecm_pid = ((data[2] & 0x1f) << 8) | data[3];
				e->id = (data[j + 2] << 16) | (data[j + 3] << 8) | (data[j + 4] & 0xf0);
				e->next = ecminfos;
				ecminfos = e;
				fprintf(stderr, "Found viaccess id %04x\n", e->id);
				}

			j += 2 + data[j + 1];
			}
		break;
	case IRDETO_CA_SYSTEM:
	case BETA_CA_SYSTEM:
		e = malloc(sizeof(struct ECMINFO));
		if (e == 0) return;
		e->system = ca_system;
		e->name = irdeto_name;
		e->ecm_pid = ((data[2] & 0x1f) << 8) | data[3];
		e->next = ecminfos;
		ecminfos = e;
		fprintf(stderr,"Found irdeto\n");
		break;
	case NAGRA_CA_SYSTEM:
		e = malloc(sizeof(struct ECMINFO));
		if (e == 0) return;
		e->system = ca_system;
		e->name = nagra_name;
		e->ecm_pid = ((data[2] & 0x1f) << 8) | data[3];
		e->next = ecminfos;
		ecminfos = e;
		fprintf(stderr,"Found nagra\n");
		break;
	case CONAX_CA_SYSTEM:
		e = malloc(sizeof(struct ECMINFO));
		if (e == 0) return;
		e->system = ca_system;
		e->name = conax_name;
		e->ecm_pid = ((data[2] & 0x1f) << 8) | data[3];
		e->next = ecminfos;
		ecminfos = e;
		fprintf(stderr,"Found conax\n");
		break;
	default:
		fprintf(stderr, "jpinfo: unknown ca_system 0x%02x\n", ca_system);
		break;
	} /* end switch ca_system */

} /* end function ParseCADescriptor */


void process()
{
int a, i;
struct ECMINFO *e;
int parity;
unsigned long packets, count;
int packets_in, packets_out;
unsigned char *inptr, *outptr;
int pid;
int payload_unit_start_indicator, pmt_length;
unsigned char *pmtbptr;
int length, info_len, data_len, index;
int pnr = -1;
int section_number;
int last_section_number;
int current_next_indicator;
int pcr_pid;
int program_info_length;
int qpid;
FILE *fptr_pf;
char temp[4096];
int have_reported_success_flag;

e = malloc(sizeof(struct ECMINFO));
if (e == 0) return;

e->ecm_pid = ecm_pid;
e->next = ecminfos;
ecminfos = e;

e = ecminfos;

have_reported_success_flag = 0;
parity = 0xff;
packets = 0;
count = 0;
packets_out = 0;
outptr = output_buffer;
while(1)
	{
	a = fread(input_buffer, 1, process_packets * 188, stdin);
	if(a != process_packets * 188) 
		{
		exit(1);
		}

	packets_in = 0;
	inptr = input_buffer;
	while(1)
		{
//		fprintf(stderr, "inptr[0]=%02x\n", inptr[0]);

		if(inptr[0] == 0x47)
			{
			pid = ( (inptr[1] & 0x1f) << 8) | inptr[2];


/*
fprintf(stderr, "WAS pid=%d =================================================\n", pid);
if(pid == 33) 
	{
	fprintf(stderr, "WAS======================================================\n");

	exit(0);
	}
*/

/*
we have 4 bytes:
8	bits	0x47 sync byte
1	bit		transport error flag
1	bit		payload unit start indicator
1	bit		transport priority
13	bits	PID
2	bits	scrambling control
2	bits	adaption field control
4	bits	continuity counter
?	bits	possible adaption field	
*/


//#define TRACE_PACKETS
#ifdef TRACE_PACKETS
trerr = (inptr[1] & 0x80) >> 7;
pstrt = (inptr[1] & 0x40) >> 6;
prio = (inptr[1] & 0x20) >> 5;
//pid = ((inptr[1] & 0x1f) << 8) | inptr[2];
src = (inptr[3] & 0xc0) >> 6;
adap = (inptr[3] & 0x20) >> 4;
cont = inptr[3] & 0x0f;

fprintf(stderr,\
"%02x %02x %02x %02x %02x %02x %02x %02x %02x %02x trerr=%d pstrt=%d prio=%d pid=%d src=%d adap=%d cont=%d\n",\
inptr[0], inptr[1], inptr[2], inptr[3], inptr[4], inptr[5], inptr[6], inptr[7] ,inptr[8], inptr[9],\
trerr, pstrt, prio, pid, src, adap, cont);

#endif
			if(! have_pids_flag)
				{
				if(! pmt_pid)
					{
					if(pid == 0) // PAT, program number gives PMT pid
						{
						int seclen;
						int p;

						fprintf(stderr, "pid 0, using program_number=%d\n", program_number);

						/* The PAT is supposed to fit in a 184 bytes packet */

						seclen = ( (inptr[1 + 5] & 0x0f) << 8) | (inptr[2 + 5] & 0xff);
						fprintf(stderr, "PAT seclen=%d\n", seclen);

						seclen += 3;
						fprintf(stderr, "Read %d bytes - Found %d services\n", seclen, (seclen - 11) / 4);

						i = 8;
						while(i < seclen - 4)
							{
							p = (inptr[i + 5] << 8) | inptr[i + 1 + 5];

							fprintf(stderr, "Found program number %d\n", p);

							if(p == program_number)
								{
								pmt_pid = ( (inptr[i + 2 + 5] & 0x1f) << 8) | inptr[i + 3 + 5];

								break;
								}

							i += 4;
							} /* end while */

						if(pmt_pid == 0)
							{
							fprintf(stderr, "jpinfo: program number %d not in PAT, stop.\n", program_number);

							exit(1);
							}
						else
							{
							fprintf(stderr,"Program number: %d PMT pid: %d (0x%x)\n",\
							program_number, pmt_pid, pmt_pid);
							}

						} /* end if pid == 0, PAT */
					} /* end if ! pmt_pid */
				else /* have pmt_pid */
					{
					if(pid == pmt_pid)
						{
						// PMT gives ECM pid

						/* we will be here again if we do not find an ecm pid in this packet */

						if(debug) fprintf(stderr, "pid = pmt_pid=%d\n", pmt_pid);

						/* test if first PMT packet (payload unit start indicator set) */

						payload_unit_start_indicator = (inptr[1] & 0x40) >> 6;

						if(debug) fprintf(stderr, "payload_unit_start_indicator=%d\n",\
									payload_unit_start_indicator);

						if(payload_unit_start_indicator) /* first packet of PMT */
							{
							subtitle_channels = 0;
							audio_channels = 0;

							pmt_length = ( (inptr[1 + 5] & 0x0f) << 8) | (inptr[2 + 5] & 0xff);
							if(debug) fprintf(stderr, "PMT section length=%d\n", pmt_length);
							
							length = pmt_length;

							/* if length > 184  then the PMT extends over more then one packet */

							pnr = (inptr[3 + 5] << 8) + inptr[4 + 5];
							if(debug) fprintf(stderr, "pnr=%d\n", pnr);

							current_next_indicator = inptr[5 + 5] & 0x1;
							if(debug) fprintf(stderr, "PMT current_next_indicator=%d\n", current_next_indicator);

							section_number = inptr[6 + 5];
							if(debug) fprintf(stderr, "PMT section_number=%d\n", section_number);						

							last_section_number = inptr[7 + 5];
							if(debug) fprintf(stderr, "PMT last_section_number=%d\n", last_section_number);						

							pcr_pid = (inptr[8 + 5] & 0x1f << 8) | (inptr[9 + 5] & 0xff);
							if(debug) fprintf(stderr, "PMT pcr_pid=%d\n", pcr_pid);	

							program_info_length = (inptr[10 + 5] & 0x0f << 8) | (inptr[11 + 5] & 0xff);
							if(debug) fprintf(stderr, "PMT program_info_length=%d\n", program_info_length);

							/* point to start PMT buffer */
							pmtbptr = pmt_buffer;
							if(pmt_length > 188 - 5) a = 188 - 5;
							else a = pmt_length;

							memcpy(pmt_buffer, inptr + 5, a);
							pmtbptr += a;
							pmt_length -= a;
							} /* end if payload_unit_start_indicator */
						else /* not first packet of PMT */
							{

							if(pmt_length > 184) a = 184;
							else a = pmt_length;

							memcpy(pmtbptr, inptr + 4, a);
							pmtbptr += a;
							pmt_length -= a;
							}

						if(debug) fprintf(stderr, "pmt_length=%d\n", pmt_length);

						if(pmt_length == 0) // all read
							{
							index = 10;

							info_len = ((pmt_buffer[index] & 0x0F) << 8) + pmt_buffer[index + 1];

							if(debug) fprintf(stderr, "PMT info_len=%d\n", info_len);

							index += 2;

							while(info_len > 0)
								{
								if (pmt_buffer[index] == 0x09)
									{
									ParseCADescriptor(&pmt_buffer[index + 2], pmt_buffer[index + 1]);
									}

								info_len -= 2 + pmt_buffer[index + 1];
								index += 2 + pmt_buffer[index + 1];
								}

							while(index < length - 4)
								{
								qpid = ((pmt_buffer[index + 1] & 0x1f) << 8) + pmt_buffer[index + 2];
								if(debug) fprintf(stderr, "qpid=%d ", qpid);
								if(debug) fprintf(stderr, "type=0x%02x ", pmt_buffer[index]);

								/* video is 1 and 2, audio is 3 and 4, teletext is 5 and 6, dvd subs is .. */
								
								switch(pmt_buffer[index])
									{
									case 1:
									case 2:
										if(debug) fprintf(stderr, "type video mpeg2\n"); 
										if(video_pid == 0)
											{
											video_pid = qpid;

											fprintf(stderr,\
											"pid %d (0x%x) video type is mpeg2\n",\
											video_pid, video_pid);
											}
										break;
									case 3:
									case 4:
										if(debug) fprintf(stderr, "type audio\n");
										if(selected_audio_pid == 0) selected_audio_pid = qpid;

										audio_pid[audio_channels] = qpid;
										audio_channels++;
										break;
									case 5: 
										switch(pmt_buffer[index + 5]) // private stream type
											{
											case 0x0e: // BBC subtititles
												subtitle_pid[subtitle_channels] = qpid;
												subtitle_channels++;
												if(debug) fprintf(stderr, "subtitle\n");   
												break;
											default:
												fprintf(stderr,\
									"found unknown stream in pid %d (0x%x), type 0x%x, sub type 0x%x.\n",\
												qpid, qpid, pmt_buffer[index], pmt_buffer[index + 5]);
												break;	
											} /* end switch pmt_buffer[index + 5] */
										break;
									case 6:
										// private streams
										if(debug) fprintf(stderr, "private stream type=0x%02x ",\
													pmt_buffer[index + 5]);
										switch(pmt_buffer[index + 5]) // private stream type
											{
											case 0x52: // ARD tetelext!
											case 0x56:
											case 0x0e: // BBC teletext
												if(teletext_pid == 0) teletext_pid = qpid;
												if(debug) fprintf(stderr, "teletext\n");   
												break;
											case 0x59:
												subtitle_pid[subtitle_channels] = qpid;
												subtitle_channels++;
												if(debug) fprintf(stderr, "subtitle\n");   
												break;
											case 0x0a: // ZDF astra1 Dolby digital
											case 0x6a:
												if(ac3_pid == 0) ac3_pid = qpid;	
												fprintf(stderr, "found ac3 audio channel pid %d\n", qpid);   
												break;
											/* see en_300468v010601o.pdf */
											case 0x53: // CA_indentifier_descriptor
											case 0x5f: // private data
											case 0x64: // data broadcast
											case 0x65: // scrambling
											case 0x66: // data_broadcast_id
											case 0x69: // PDC_descriptor
											default:
												fprintf(stderr,\
									"found unknown stream in pid %d (0x%x), type 0x%x, sub type 0x%x.\n",\
												qpid, qpid, pmt_buffer[index], pmt_buffer[index + 5]);
												break;
											} /* end switch pmt_buffer[index + 5] */
										break;
//									case 11:
//										switch(pmt_buffer[index + 5]) // private stream type
//											{
//											default:

/*												fprintf(stderr,\
									"found unknown stream in pid %d (0x%x), type 0x%x, sub type 0x%x.\n",\
												qpid, qpid, pmt_buffer[index], pmt_buffer[index + 5]);
*/
//												break;
//											}
//										break;
									case 27: // mpeg4 H264
										if(debug) fprintf(stderr, "type video mpeg4 H264\n"); 
										if(video_pid == 0)
											{
											video_pid = qpid;

											fprintf(stderr,\
											"pid %d (0x%x) video type is mpeg4 H264\n",\
											video_pid, video_pid);
											}
										
										break;
									default:
										fprintf(stderr, "pid=%d pid type=%d (0x%02x), dunno what pid this is.\n",\
										qpid, pmt_buffer[index], pmt_buffer[index]);
										break;
									} /* end switch pid type */

								data_len = ((pmt_buffer[index + 3] & 0x0F) << 8) + pmt_buffer[index + 4];
								if (pmt_buffer[index] == 0x02 || pmt_buffer[index] == 0x03 || pmt_buffer[index] == 0x04)
									{
									i = index + 5;
									while (i < index + 5 + data_len)
										{
										switch (pmt_buffer[i])
											{
										 	case 0x0a:
												if (pmt_buffer[index] == 3 || pmt_buffer[index] == 4)
													{
													fprintf(stderr,\
													"Language = %.3s pid %d (0x%x)\n",\
													&pmt_buffer[i + 2], qpid, qpid);

													if(memcmp(&pmt_buffer[i + 2], language, 3) == 0)
															selected_audio_pid = qpid;
													}
												break;
											case 0x09:
												ParseCADescriptor(&pmt_buffer[i + 2], pmt_buffer[i + 1]);
												break;
											} /* end switch */

										i += 2 + pmt_buffer[i + 1];
										}
									}
								index += 5 + data_len;
								}

							fprintf(stderr, "setting PIDS for program number %d (0x%x):\n", pnr, pnr);

							if(video_pid) fprintf(stderr, " video_pid=%d (0x%x)\n", video_pid, video_pid);
							else fprintf(stderr, " no video pid found\n");

							if(use_ac3_pid_only_flag && ac3_pid)
								{
								fprintf(stderr, " using ac3 pid %d only\n", ac3_pid);	
								}
							else
								{
								if(audio_channels)
									{
									fprintf(stderr,\
									" found %d audio channels, pid: ", audio_channels);
									for(i = 0; i < audio_channels; i++)
										{
										fprintf(stderr, "%d (0x%x) ", audio_pid[i], audio_pid[i]);
										}
									fprintf(stderr, "\n");
								
									if(strcmp(language, "+") == 0)
										{
										fprintf(stderr, "passing all %d audio channels\n", audio_channels);
										}
									else
										{
										if(selected_audio_pid)
											{
											fprintf(stderr, " selected_audio_pid=%d (0x%x) language=%s\n",\
											selected_audio_pid, selected_audio_pid, language);
											}
										else fprintf(stderr, " no audio pid found for language %s\n", language);
										}
									}
								else fprintf(stderr, " no audio channels found\n");
								}

							if(ac3_pid) fprintf(stderr, " ac3 pid=%d\n", ac3_pid);
							else fprintf(stderr, " no ac3 pid found\n");

							if(teletext_pid) fprintf(stderr, " teletext_pid=%d (0x%x)\n",\
							teletext_pid, teletext_pid);
							else fprintf(stderr, " no teletext pid found\n");

							if(subtitle_channels)
								{
								fprintf(stderr,\
								" found %d subtitle channels, pid: ", subtitle_channels);
								for(i = 0; i < subtitle_channels; i++)
									{
									fprintf(stderr, "%d (0x%x) ", subtitle_pid[i], subtitle_pid[i]);
									}
								fprintf(stderr, "\n");
								}
							else fprintf(stderr, " no subtitle channels found\n");

							if(ecminfos->ecm_pid)
								{
								fprintf(stderr, " initial_ecm_pid=%d (0x%x)\n",\
								ecminfos->ecm_pid, ecminfos->ecm_pid);
								}
							else
								{
								fprintf(stderr, " no ecm pids found\n");
								}
						
							/* write pids found to ~/jpinfo_pid */
							sprintf(temp, "%s/jpcam_pids", home_dir);
							fptr_pf = fopen(temp, "w");
							if(! fptr_pf)
								{
								fprintf(stderr, "jpinfo: could not open file %s for write, ", temp); 	
								perror("reason is ");

								exit(0);
								}
							else
								{
								fprintf(fptr_pf, "video_pid %d\n", video_pid);	
								if(audio_channels)
									{
									if(strcmp(language, "+") == 0)
										{
										for(i = 0; i < audio_channels; i++)
											{
											fprintf(fptr_pf, "audio_pid %d\n", audio_pid[i]);
											}
										}
									else
										{
										fprintf(fptr_pf, "selected_audio_pid %d\n", selected_audio_pid);							
										}
									}
									
								if(ac3_pid) fprintf(fptr_pf, "ac3_pid %d\n", ac3_pid);

								if(teletext_pid) fprintf(fptr_pf, "teletext_pid %d\n", teletext_pid);							

								for(i = 0; i < subtitle_channels; i++)
									fprintf(fptr_pf, "subtitle_pid %d\n", subtitle_pid[i]);

								if(ecminfos->ecm_pid)
									{
									fprintf(fptr_pf, "initial_ecm_pid=%d\n", ecminfos->ecm_pid);
									}

								fclose(fptr_pf);
								} /* end if ~/jpcam_pid open */

							have_pids_flag = 1;
							} /* end if pmt complete */

						} /* end if pid == PMT pid */
					} /* end if have pmt_pid */

				e = ecminfos;

				} /* end if ! have_pids_flag */

			/* test if ecm pid */
			if( (pid != 0) && (pid == e->ecm_pid) )
				{
				/* test if scrambled */
				if( (inptr[5] == 0x80) || (inptr[5] == 0x81) )
					{
					if(parity != (inptr[5] & 1) )
						{
						scrambled_flag = 1;					
						fprintf(stderr, "transmisison seems scrambled\n");
//						exit(1);
						}
					}
				
				} /* end if ecm pid */

			a = 0;
			if(pass_all_flag) a = 1;

			if(pid == pmt_pid) a = 1; // for offline decoding

			// all ecm pids for offline decoding
			for(pa = e; pa != 0; pa = pa->next)
				{
				if(pid == pa->ecm_pid) a = 1;
				}	

			if(pid == video_pid) a = 1;

			if(pid == teletext_pid) a = 1;

			if(use_ac3_pid_only_flag && ac3_pid)
				{
				if(pid == ac3_pid) a = 1;
				}
			else
				{
				if(strcmp(language, "+") == 0)
					{
					for(i = 0; i < audio_channels; i++) if(pid == audio_pid[i]) a = 1;

					if(pid == ac3_pid) a = 1;
					}
				else
					{
					if(pid == selected_audio_pid) a = 1;
					}
				}

			for(i = 0; i < subtitle_channels; i++) if(pid == subtitle_pid[i]) a = 1;

			if(a)
 				{
				memcpy(outptr, inptr, 188);
				outptr += 188;
				packets_out++;

				if(packets_out == csa_packets)
					{
					a = fwrite(output_buffer, 1, csa_packets * 188, stdout);
					if(a != csa_packets * 188)
						{
						fprintf(stderr, "jpinfo: could not write to stdout, aborting.\n");
			
						exit(1);		
						}
					packets_out = 0;
					outptr = output_buffer;
					}

				packets++;

				} /* end pid != ecm pid */

			} /* end if inptr[0] == 0x47 */ 
		else /* not a TS packet */
			{
			fprintf(stderr, "Packet %lu - not a TS!\n", count);
			}

		count++;

		inptr += 188;

		packets_in++;
		if(packets_in == process_packets) break;
		} /* end while read 188 bytes */

	} /* end while fill buffer */

} /* end function process */
	


int print_usage()
{
fprintf(stderr, "\
usage: cat transponder.ts | jpinfo [-a] [-b] [-d] [-f] [-h] [-i] [-l language] -p program_pid > program.ts\n\
-a  let all packets through (default: video, audio, ac3, teletext, and subtitle pids for selected program).\n\
-b  use ac3 audio pid only.\n\
-d  print debug info.\n\
-f  fast mode, uses large buffers, this will cause problems on players with small buffers.\n\
-h  help (this help).\n\
-i  info mode, prints info about program, then exits.\n\
     use -p 65535 for all program numbers in the stream.\n\
-l  language, the first 3 characters, in lower case. Use  -l +  to pass all audio pids for this program.\n\
-p  program number (in decimal, or hex 0xabc).\n\
");

return 1;
} /* end function print_usage */



int main(int argc, char **argv)
{
int a;
int fast_flag;

/* defaults */
debug = 0;
process_packets = PROCESS_PACKETS;
csa_packets = CSA_PACKETS;
fast_flag = 0;
pass_all_flag = 0;
info_only_flag = 0;
use_ac3_pid_only_flag = 0;

strcpy(language, DEFAULT_LANGUAGE);

/* end defaults */

fprintf(stderr, "\njpinfo-%s By Jan Pantelje\n", VERSION);

if(argc < 2)
	{
	print_usage();
	exit(1);
	}

while(1)
	{
	a = getopt(argc, argv, "abdfhil:np:");
	if(a == -1) break;

	switch(a)
		{
		case 'a':
			pass_all_flag = 1;
			break;
		case 'b':
			use_ac3_pid_only_flag = 1;
			break;
		case 'd':
			debug = 1;
			break;
		case 'f':
			fast_flag = 1;
			break;
		case 'h':
			print_usage();
			exit(1);
			break;
		case 'i':
			info_only_flag = 1;
			break;
		case 'l':
			strncpy(language, optarg, 3);
			break;
		case 'p':
			sscanf(optarg, "%i", &program_number);
			break;
		default:
			print_usage();
			exit(1);
			break;
		}/* end switch a */
	} /* end while getopt() */

/* get user info */ 
userinfo = getpwuid(getuid() );

/* get home directory */
home_dir = strsave(userinfo -> pw_dir);
if(! home_dir)
    {
    fprintf(stderr,\
    "jpinfo: could allocte space for home_dir\n");
    exit(1);
    }

user_name = strsave(userinfo -> pw_name);
if(! user_name)
    {
    fprintf(stderr,\
    "jpinfo: could allocate space for user_name\n");
    exit(1);
    }

fprintf(stderr,\
"jpinfo: program_number=%d pass_all_flag=%d language=%s fast_flag=%d info_only_flag=%d debug=%d\n",\
program_number, pass_all_flag, language, fast_flag, info_only_flag, debug);

if(fast_flag)
	{
	process_packets *= 10;
	csa_packets *= 10;
	}

if(program_number == 0) 
	{
	fprintf(stderr, "jpinfo: specify a program number with -p 1234.\n");

	exit(1);
	}

input_buffer = (uint8_t *)malloc(process_packets * 188);
if(! input_buffer)
	{
	fprintf(stderr, "jpinfo: could not allocate space for input_buffer, aborting.\n");

	exit(1);
	}

output_buffer = (uint8_t *)malloc(csa_packets * 188);
if(! output_buffer)
	{
	fprintf(stderr, "jpinfo: could not allocate space for output_buffer, aborting.\n");

	exit(1);
	}

process();

exit(0);
} /* end function main */


